@@ -294,6 +294,7 @@ class IterativeAutoregressiveTrainer(AutoregressiveTrainer):
             fp8_history_len: int = 256,
             fp8_margin: int = 0,
             collect_log_interval: int = 100,
+            use_iterable_dataset: bool = True,
             **kwargs
     ):
         super().__init__(
@@ -306,7 +307,7 @@ class IterativeAutoregressiveTrainer(AutoregressiveTrainer):
             moe_aux_loss_scale=moe_aux_loss_scale,
             use_f1_metrics=use_f1_metrics,
             is_sft=is_sft,
-            use_iterable_dataset=True,
+            use_iterable_dataset=use_iterable_dataset,
             **kwargs
         )
         self.collect_n_batches = collect_n_batches
